r"""
Features for testing the presence of ``tdlib``
"""

# *****************************************************************************
#       Copyright (C) 2021 Matthias Koeppe
#                     2021 Kwankyu Lee
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import PythonModule
from .join_feature import JoinFeature


class Tdlib(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of the SageMath interface to the :ref:`tdlib <spkg_tdlib>` library.
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.tdlib import Tdlib
            sage: isinstance(Tdlib(), Tdlib)
            True
        """
        # Currently part of sagemath_standard, conditionally built.
        # Will be changed to spkg='sagemath_tdlib' later
        JoinFeature.__init__(self, 'tdlib',
                             [PythonModule('sage.graphs.graph_decompositions.tdlib', spkg='tdlib')])


def all_features():
    return [Tdlib()]



import os

from . import StaticFile


class Threejs(StaticFile):
    r"""
    A :class:`~sage.features.Feature` which describes the presence of
    threejs-sage in a few standard locations.

    EXAMPLES::

        sage: from sage.features.threejs import Threejs
        sage: bool(Threejs().is_present())  # needs threejs
        True
    """

    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.threejs import Threejs
            sage: isinstance(Threejs(), Threejs)
            True
        """
        from sage.env import SAGE_SHARE, THREEJS_DIR

        threejs_search_path = THREEJS_DIR or (
            os.path.join(SAGE_SHARE, "jupyter", "nbextensions", "threejs-sage"),
            os.path.join(SAGE_SHARE, "sagemath", "threejs-sage"),
            os.path.join(SAGE_SHARE, "sage", "threejs"),
            os.path.join(SAGE_SHARE, "threejs-sage")
            )

        try:
            version = self.required_version()
            filename = os.path.join(version, "three.min.js")
        except FileNotFoundError:
            filename = 'unknown'

        StaticFile.__init__(
            self, name="threejs",
            filename=filename,
            spkg="threejs",
            type="standard",
            search_path=threejs_search_path,
            description="JavaScript library to display 3D graphics")

    def required_version(self):
        """
        Return the version of threejs that Sage requires.

        Defining what version is required is delegated to the distribution package
        that provides the file ``threejs-version.txt`` in :mod:`sage.ext_data.threejs`.

        If the file is not provided, :class:`FileNotFoundError` is raised.

        EXAMPLES::

            sage: from sage.features.threejs import Threejs
            sage: Threejs().required_version()
            'r...'
        """
        from sage.env import SAGE_EXTCODE

        filename = os.path.join(SAGE_EXTCODE, 'threejs', 'threejs-version.txt')

        with open(filename) as f:
            return f.read().strip()


def all_features():
    return [Threejs()]



r"""
Features for testing the presence of ``cython``
"""

# *****************************************************************************
#       Copyright (C) 2021 Matthias Koeppe
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from . import CythonFeature


class sage__misc__cython(CythonFeature):
    r"""
    A :class:`~sage.features.Feature` which describes whether :mod:`sage.misc.cython`
    is available and functional.
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features import CythonFeature
            sage: from sage.features.cython import sage__misc__cython
            sage: isinstance(sage__misc__cython(), CythonFeature)
            True
        """
        # It suffices to use a trivial CythonFeature because CythonFeature
        # is implemented via sage.misc.cython.
        CythonFeature.__init__(self, "sage.misc.cython", test_code="")


def all_features():
    return [sage__misc__cython()]



r"""
Feature for testing the presence of ``imagemagick``

Currently we only check for the presence of ``convert``. When needed, other
commands like ``magick``, ``magick-script``, ``convert``, ``mogrify``,
``identify``, ``composite``, ``montage``, ``compare``, etc. could be also
checked in this module.
"""

# ****************************************************************************
#       Copyright (C) 2018-2022 Sebastien Labbe <slabqc@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from . import Executable, FeatureTestResult
from .join_feature import JoinFeature

class Convert(Executable):
    r"""
    A :class:`~sage.features.Feature` describing the presence of ``convert``.

    EXAMPLES::

        sage: from sage.features.imagemagick import Convert
        sage: Convert().is_present()  # optional - imagemagick
        FeatureTestResult('convert', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.imagemagick import Convert
            sage: isinstance(Convert(), Convert)
            True
        """
        Executable.__init__(self, "convert", executable="convert")

    def is_functional(self):
        r"""
        Return whether command ``convert`` in the path is functional.

        EXAMPLES::

            sage: from sage.features.imagemagick import Convert
            sage: Convert().is_functional()   # optional - imagemagick
            FeatureTestResult('convert', True)

        """
        # Create the content of 1-pixel png file
        content = b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x01\x00\x00\x00\x01\x08\x00\x00\x00\x00:~\x9bU\x00\x00\x00\nIDATx\x9cc`\x00\x00\x00\x02\x00\x01H\xaf\xa4q\x00\x00\x00\x00IEND\xaeB`\x82'

        # NOTE:
        #
        # This is how the above content of a 1 pixel PNG was created::
        #
        #    sage: import numpy as np
        #    sage: from PIL import Image
        #    sage: image = Image.fromarray(np.array([[100]], dtype=np.uint8))
        #    sage: image.save('file.png')
        #    sage: with open('file.png', 'rb') as f:
        #    ....:     content = f.read()

        # create a png file with the content
        from sage.misc.temporary_file import tmp_filename
        base_filename_png = tmp_filename(ext='.png')
        with open(base_filename_png, 'wb') as f:
            f.write(content)

        # Set up filenames
        import os
        base, filename_png = os.path.split(base_filename_png)
        filename, _png = os.path.splitext(filename_png)
        filename_gif = filename + '.gif'

        # running command convert (taken from sage/plot/animate.py)
        from subprocess import run
        cmd = ['convert', '-dispose', 'Background', '-delay', '20',
                '-loop', '0', filename_png, filename_gif]

        try:
            result = run(cmd, cwd=base, capture_output=True, text=True)
        except OSError as e:
            return FeatureTestResult(self, False, reason='Running command "{}" '
                        'raised an OSError "{}" '.format(' '.join(cmd), e))

        # If an error occurred, return False
        if result.returncode:
            return FeatureTestResult(self, False, reason='Running command "{}" '
                        'returned non-zero exit status "{}" with stderr '
                        '"{}" and stdout "{}".'.format(result.args,
                                                       result.returncode,
                                                       result.stderr.strip(),
                                                       result.stdout.strip()))

        # If necessary, run more tests here
        # ...

        # The command seems functional
        return FeatureTestResult(self, True)


class ImageMagick(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` describing the presence of
    :ref:`ImageMagick <spkg_imagemagick>`

    Currently, only the availability of the :class:`convert` program is checked.

    EXAMPLES::

        sage: from sage.features.imagemagick import ImageMagick
        sage: ImageMagick().is_present()  # optional - imagemagick
        FeatureTestResult('imagemagick', True)
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.imagemagick import ImageMagick
            sage: isinstance(ImageMagick(), ImageMagick)
            True
        """
        JoinFeature.__init__(self, "imagemagick",
                             [Convert()],
                             spkg="imagemagick",
                             url="https://www.imagemagick.org/")

def all_features():
    return [ImageMagick()]
